/*
 * lfo.h
 *
 *  Created on: Aug 22, 2024
 *      Author: sei_s
 */

#ifndef INC_LFO_H_
#define INC_LFO_H_

#include "main.h"
#include "synth.h"
#include "osc.h"

/*===============================================================
     definition
===============================================================*/

//
// LFO maximum data macro
//
#define LFO_MAX_FREQ      20.0F    // max 20[Hz]
#define LFO_MIN_FREQ       0.0F    // min  0[Hz]
#define LFO_MAX_AMP        1.0F    //


/*===============================================================
     global variable
===============================================================*/

extern volatile int          lfo_wave;       // SINE_WAVE, SAW_WAVE, PULSE_WAVE
extern volatile const float *lfo_wave_select[];
extern volatile const float *lfo_wave_table; // lfo_wave_select[lfo_wave]
// LFO #1
extern volatile float        lfo1_freq;       // 0Hz - LFO_MAX_FREQ
extern volatile float        lfo1_delta;      // phase increase step  lfo_freq / SAMPLING_FREQ
extern volatile float        lfo1_theta;      // current pahse
extern volatile float        lfo1_level;      // output level   0.0 < 1.0
// LFO #2
extern volatile float        lfo2_freq;       // 0Hz - LFO_MAX_FREQ
extern volatile float        lfo2_delta;      // phase increase step  lfo_freq / SAMPLING_FREQ
extern volatile float        lfo2_theta;      // current pahse
extern volatile float        lfo2_level;      // output level   0.0 < 1.0

extern volatile int          lfo1_trig;       // 1= eg trigger on, 0= eg trigger off
extern volatile int          lfo2_trig;       // 1= eg trigger on, 0= eg trigger off
extern volatile MIDI_NOTE   *rpt_note;        // repeat_play voice


/*===============================================================
    function prototype
===============================================================*/

extern void         lfo_init(void);
extern void         lfo_process(void);
extern void         lfo_repeat_play(void);


#endif /* INC_LFO_H_ */
/*** end of file ***********************************************/
