/*
 * osc.h
 *
 *  Created on: Aug 22, 2024
 *      Author: sei_s
 */

#ifndef INC_OSC_H_
#define INC_OSC_H_

#include "synth.h"

/*===============================================================
     definition
===============================================================*/

//
//  wave type macro
//
#define SINE_WAVE         0
#define SAW_WAVE          1
#define PULSE_WAVE        2
#define NOISE_WAVE        3

//
// KEY PORTAMENTO
//
#define MAX_PORTAMENT     500
#define MIN_PORTAMENT     1


/*===============================================================
     global variable
===============================================================*/

//
//  wave data table
//
extern const float        sine_wave_table[1024];
extern const float        saw_wave_table[5][1024];
extern const float        pulse_wave_table[5][1024];
extern float              use_wave_table[5];

//
// wave select
//
extern int                vco_wave;

//
// FM modulation
//
extern volatile float     fm_ratio;
extern volatile float     fm_depth;
//
// PORTAMENT
//
extern float              vco_prev_delta;
extern int                vco_prev_note;
extern volatile int       vco_portament;
extern volatile int       vco_portament1;

//
//  pitch control
//
extern volatile float     vco_pitch;          // +/- 1octave
extern volatile float     vco_pitch_shift;  // chorus 1.000 - 1.005  max 0.5% shift
extern volatile float     vco_lfo1_mod_depth;
extern volatile float     vco_lfo2_mod_depth;
extern volatile float     vco_eg1_mod_depth;
extern volatile float     vco_eg2_mod_depth;

//
// amplitude control
//
extern volatile float     vco_lfo1_tremoro_depth;
extern volatile float     vco_lfo2_tremoro_depth;

//
//  efect control
//
extern volatile int       vco_chorus;


/*===============================================================
    function prototype
===============================================================*/

extern void  osc_init(void);
extern float osc_process(MIDI_NOTE *note);

#endif /* INC_OSC_H_ */
/*** end of file ***********************************************/

