/********************************************************************
 * sd_file.h
 *
 *  Created on: Oct 7, 2024
 *      Author: sei_s
 *
 *
 *
 * << File Data Format >>
 *
 *  Marker + Data, Marker + Data,
 *
 *  Marker
 *     0: pause time        32bit [ms]
 *        Data size 4otet
 *
 *     1: Midi Note ON/OFF  09,90,note,velocity
 *        Data size 4octet  08,80,note,velocity
 *
 *     2: fm operator
 *        Data size sizeof(operator)
 *
 *     3: fm synth unit
 *        Data size sixeof(synth)
 *
 *   255: passed file end
 *
 *******************************************************************/

#ifndef INC_SD_FILE_H_
#define INC_SD_FILE_H_

#include "synth.h"

/*====================================================================
    definition
====================================================================*/

#define  PAUSE_DATA            1
#define  MIDI_NOTE_DATA        2
#define  PARAMETER_DATA        3
#define  DATA_END_MARK         255

#define  PAUSE_DATA_SIZE       4
#define  MIDI_NOTE_DATA_SIZE   4
#define  PARAMETER_DATA_SIZE   sizeof(PARAM)

#define  MAX_FILES             30
#define  FILE_NAME_SIZE        36

/*====================================================================
    function prototype
====================================================================*/

extern void dir_init(void);
extern void read_dir_entry(void);
extern void write_dir_entry(void);
extern void display_file_list(void);
extern void delete_file(uint32_t file_number);

extern void begin_recording(uint32_t file_number, char *fname);
extern void data_recording(void *data, uint32_t data_size);
extern void end_recording(void);

extern int  begin_playing(uint32_t file_number);
extern void data_playing(void *data, uint32_t data_size);
extern void end_playing(void);

extern void sdcard_test(void);

#endif /* INC_SD_FILE_H_ */
/*** end of file *****************************************************/
