/***************************************************************
 * synth.h
 *
 *  Created on: Aug 22, 2024
 *      Author: sei_s
 **************************************************************/

#ifndef INC_SYNTH_H_
#define INC_SYNTH_H_

/*===============================================================
     definition
===============================================================*/
// type define
//
// note parameter EG
//
typedef struct {
	int   state;
    float level;
} EG;


// note parameter VCF
typedef struct {
	float x1, x2, y1, y2, x0, y0;
} VCF;

typedef struct {
	int   use;
	int   numb;
	const float *wave_table;
    float freq;
	float delta;
	float theta;
	float velocity;
	float portament_ratio;  // portament time
	int   portament;        // 2*(prev_note_numb-note_numb)
    int   sampl_cnt;        // work of portament
    int   state;
    int   pitch_shift;      // for chorus
    VCF   lpf;              // low pass filter
    VCF   hpf;              // high pass filter
	EG    eg[2];               // two envelope generator
} MIDI_NOTE;

// number of poliphonics
#define MAX_NOTE          5


typedef struct {
    int      use;
    int      pot_val[20];
    int      key_switch[11]; // execept menu key(SET/UP/DOWN)
    int      vco_wave;
    int      lfo_wave;
} PARAM;

// number of parameter
#define PARAM_MAX        10


#define PI                ((float) M_PI)   // float型の円周率
#define SAMPLING_FREQ     (40000)          // [Hz] sampling frequency

//
// modulation parameter
//
#define  MOD_MIN_DEPTH     (0.0F)      // 0%
#define  MOD_MAX_DEPTH     (0.999F)    // 99.9%
#define  MOD_DEPTH_STEP    (0.005F)    // 0.5%

//
// play mode
//
#define  NORMAL_MODE       0
#define  FIXED_MODE        1



/*===============================================================
     global variable
===============================================================*/

extern const     float    note_freq[136];
extern MIDI_NOTE          midi_note[MAX_NOTE];

#endif /* INC_SYNTH_H_ */
/*** end of file ***********************************************/
