/***************************************************************
 * echo.c
 *
 *  Created on: Aug 22, 2024
 *      Author: sei_s
 **************************************************************/

#include "synth.h"
#include "echo.h"
#include "uart2.h"

float  cf1[1997];
float  cf2[2803];
float  cf3[3559];
float  af1[97];
float  af2[131];
int    cf1_ptr;
int    cf2_ptr;
int    cf3_ptr;
int    af1_ptr;
int    af2_ptr;

float  echo_depth;
int    echo_time;   // echo_time=1(echo on), =0(echo off)


/*===============================================================
    echo initialize
===============================================================*/

void echo_init(void)
{
	echo_time = 0;
	echo_depth = 0.3F;  // 75% fix
    cf1_ptr = cf2_ptr = cf3_ptr = 0;
    af1_ptr = af2_ptr = 0;
}


/*===============================================================
    osc process
===============================================================*/

float echo_process(float in)
{
	float cf_in, af1_in, af2_in, out;
    float cf1_out, cf2_out, cf3_out;
    float af1_out, af2_out;


    if(echo_time == 0) return in;   // echo_time=1(echo on), =0(echo off)


    cf_in = in * (1.0F-echo_depth);

    cf1_out = cf1[cf1_ptr];
    cf1[cf1_ptr++] = cf_in + cf1_out * echo_depth;
    cf1_ptr %= 1997;

    cf2_out = cf2[cf2_ptr];
    cf2[cf2_ptr++] = cf_in + cf2_out * echo_depth;
    cf2_ptr %= 2803;

    cf3_out = cf3[cf3_ptr];
    cf3[cf3_ptr++] = cf_in + cf3_out * echo_depth;
    cf3_ptr %= 3559;

    af1_in  = cf1_out + cf2_out + cf3_out;
    af1_out = af1[af1_ptr];
    af1[af1_ptr]= af1_in + af1_out * echo_depth * 0.8f;
    af1_out -= (echo_depth * 0.8f)*af1[af1_ptr++];
    af1_ptr %= 97;

    af2_in = af1_out;
    af2_out = af2[af2_ptr];
    af2[af2_ptr]= af2_in + af2_out * echo_depth * 0.8f;
    af2_out -= (echo_depth * 0.8f)*af2[af2_ptr++];
    af2_ptr %= 131;

    out = (in + af2_out);

    return out;

}


/*** end of file ***********************************************/

