/*
 * eg.c
 *
 *  Created on: Aug 22, 2024
 *      Author: sei_s
 *
 * static float curr_gain = 0;
 * float        targ_gain = 0.8F;
 * curr_gain = targ_gain -((targ_gain - curr_gain) * (255.0F / 256.0F));
 * return curr_gain;
 *
 */

#include "synth.h"
#include "eg.h"
#include "lfo.h"
#include "uart2.h"

/*===============================================================
     private variable
===============================================================*/


/*===============================================================
     global variable
===============================================================*/


volatile float     eg_atack_ratio[2];     // 0.99 - 0.9999 init 0.99(aboud 10msec)
//volatile float     eg_decay_ratio;        // 0.99 - 0.9999 init 0.999(about 100msec)
//volatile float     eg_sustain_level;      // 0.0 -  0.99999 init 0.8  80%
volatile float     eg_release_ratio[2];   // 0.99 - 0.9999 init 0.9999 (about 1000msec)

volatile int       eg_ptrn[2];            // EG_AR, EG_KEYOFF, EG_ASR
volatile float     eg_vcf_mod_depth;      // 0.0 < 1.0
volatile float     eg_vco_mod_depth;      // 0.0 < 1.0


/*===============================================================
    function prototype
===============================================================*/

// global
void        eg_init(void);
void        eg_process(MIDI_NOTE *note);

// private
static void  eg_ar_process(MIDI_NOTE *note, int eg_num);
static void  eg_asr_process(MIDI_NOTE *note, int eg_num);


/*===============================================================
    Envelope Generator initilize
===============================================================*/

void eg_init(void)
{
	eg_atack_ratio[0] = EG_MIN_RATIO;
    eg_atack_ratio[1] = EG_MIN_RATIO;
	eg_release_ratio[0] = EG_MAX_RATIO;
    eg_release_ratio[1] = EG_MAX_RATIO;

    eg_ptrn[0] = EG_ASR;
    eg_ptrn[1] = EG_ASR;

	eg_vco_mod_depth = MOD_MIN_DEPTH;
	eg_vcf_mod_depth = MOD_MIN_DEPTH;
}


/*===============================================================
    Envelope Generator
===============================================================*/

inline void eg_process(MIDI_NOTE *note)
{
    switch(eg_ptrn[EG1]){
    case EG_AR:
    case EG_KEYOFF:
        eg_ar_process(note, EG1);
        break;

    case EG_ASR:
        eg_asr_process(note, EG1);
        break;
    }

    switch(eg_ptrn[EG2]){
    case EG_AR:
    case EG_KEYOFF:
        eg_ar_process(note, EG2);
        break;

    case EG_ASR:
        eg_asr_process(note, EG2);
        break;
    }

//	if(eg_select == EG_AR){
//		return eg_ar_process(note);
//
//	}else if(eg_select == EG_ADSR){
//		return eg_adsr_process(note);
//	}
//	return 1.0F;    // select KBD_GATE in amp
}

static inline void eg_ar_process(MIDI_NOTE *note, int eg_num)
{
    EG *eg;

    eg = &(note->eg[eg_num]);

    switch(eg->state){
	case ST_ATACK:
	    eg->level = 1.0F - (1.0F - eg->level)*eg_atack_ratio[eg_num];
	    if(eg->level > 0.99F){
		    eg->state = ST_RELEASE;
	    }
	    break;
	case ST_RELEASE:
	    eg->level = 0.0F - (0.0F - eg->level)*eg_release_ratio[eg_num];
	    if(eg->level < 0.001F){
	        if(rpt_note == NULL || rpt_note->numb != note->numb){
	            note->use=0;
	        }
	    }
	    break;
    }

}

static inline void eg_asr_process(MIDI_NOTE *note, int eg_num)
{
	EG *eg;

	eg = &(note->eg[eg_num]);

	switch(eg->state){
	case ST_ATACK:
		eg->level = 1.0F - (1.0F - eg->level)*eg_atack_ratio[eg_num];
		if(eg->level > 0.99F){
			eg->state = ST_SUSTAIN;
		}
		break;

	case ST_SUSTAIN:
		break;

	case ST_RELEASE:
		eg->level = 0.0F - (0.0F - eg->level)*eg_release_ratio[eg_num];
        if(eg->level < 0.001F){
            if(rpt_note == NULL || rpt_note->numb != note->numb){
                note->use=0;
            }
        }
		break;
	}

}

/*** end of file ***********************************************/
