/*
 * graphic.c
 *
 *  Created on: Aug 5, 2024
 *      Author: sei_s
 */

/*
 * gram[]
 *    seg0 seg1  ...   seg127
 *     0   　0           0            page0    0-127
 * p   1    1           1            page1  128-255
 * a   2    2           2            page2  256-383
 * g   3    3           3            page3  384-511
 * e   4    4           4            page4  512-639
 * 0   5    5           5            page5  640-767
 *     6                             page6  768-895
 *     7                             page7  896-1023
 *     0
 * p   1
 * a   2
 * g   3
 * e   4
 * 1   5
 *     6
 *     7
 *
 *     .
 *     .
 *     .
 *     0
 * p   1
 * a   2
 * g   3
 * e   4
 * 7   5
 *     6
 *     7
 *
 *
 */

#include <string.h>
#include <stdlib.h>
#include "graphic.h"
#include "uart2.h"

/*=====================================================================
    global variable
=====================================================================*/

char gram[1024];


/*=====================================================================
    grp_pset(int px, int py, int pv)

    px= 0-127
    py= 0-63
    pv= 1:on  0:off
=====================================================================*/

static int bits[]= { 1, 2, 4, 8, 16, 32, 64, 128};
void grp_pset(int px, int py, int pv)
{
	// px=0-127
	// py=0-63
	// pv=1:dot on  pv=0:dot off
    int idx;

    if(px < 0 || px > GRP_X_MAX) return;
    if(py < 0 || py > GRP_Y_MAX) return;

    py = GRP_Y_MAX - py;

	idx = (py/8)*128 +px;
    if(pv){
    	gram[idx] |= bits[py%8];
    }else{
    	gram[idx] &= ~bits[py%8];
    }
}

/*=====================================================================
    grp_clear(void)

    gram clear
=====================================================================*/

void grp_clear(void)
{
	memset(gram, 0, sizeof(gram));
}


/*=====================================================================
    grp_line(int xs, int ys, int xe, int ye, int pv)

    xs,xe 0-127
    ys,ye 0-63
    pv    0/1    1=on, 0=off
=====================================================================*/

void grp_line(int xs, int ys, int xe, int ye, int pv)
{
	int x, y, a, b;

	//UART2_putstring("grp_line()\r\n");
	//UART2_putstring("xs="); UART2_putdec(xs); UART2_putcrlf();
	//UART2_putstring("xe="); UART2_putdec(xe); UART2_putcrlf();
	//UART2_putstring("ys="); UART2_putdec(ys); UART2_putcrlf();
	//UART2_putstring("ye="); UART2_putdec(ye); UART2_putcrlf();

	a = ((ye-ys)<< 8)/(xe-xs);
	b = ((xe-xs)<< 8)/(ye-ys);

	if(abs(xe-xs) >= abs(ye-ys)){
		// x を増減
		if(xe > xs){
			for(x=xs; x<=xe; x++){
				grp_pset(x, ((a*(x-xs)+128)>>8) +ys, pv );
			}
		}else{
			for(x=xs; x>=xe; x--){
                grp_pset(x, ((a*(x-xs)+128)>>8) +ys, pv );
			}
		}
	}else{
		// y を増減
		if(ye > ys){
			for(y=ys; y <=ye; y++){
				grp_pset(((b*(y-ys)+128)>>8) +xs, y, pv );
			}
		}else{
			for(y=ys; y >=ye; y--){
				grp_pset(((b*(y-ys)+128)>>8) +xs, y, pv );
			}
		}
	}
}


/*** end of file *****************************************************/
