/************************************************************************
 * key.c
 *
 *  Created on: Oct 29, 2024
 *      Author: sei_s
 *
 ***********************************************************************/

#include <stdio.h>
#include <string.h>
#include "main.h"
#include "flash.h"
#include "uart2.h"
#include "graphic.h"
#include "oled_128x64.h"
#include "synth.h"
#include "eg.h"
#include "osc.h"
#include "lfo.h"
#include "filter.h"
#include "amp.h"
#include "echo.h"
#include "analog_synth.h"
#include "key.h"
#include "menu.h"


/*=======================================================================
    global variable
========================================================================*/

int      key_switch[14];


/*=======================================================================
    function prototype
========================================================================*/

void read_key_switch(void);
void read_menu_key(void);
void keyboard_control(void);


//static void help_key(void);


/*=======================================================================
    Key Switch control
========================================================================*/

void keyboard_control(void)
{
    static int vco_wave_select_flg = KEY_OFF;
    static int vco_chorus_flg = KEY_OFF;
    static int lfo_wave_select_flg = KEY_OFF;

    //char c;

    //*** read toggle key switch ********************
    if(play_mode == NORMAL_MODE) read_key_switch();

    //*** display initial value *********************
    //  VCO Wave
    //if(vco_wave_select_flg == -1){
    //    vco_wave_select_flg = KEY_OFF;
    //    oled_locate(0,1);
    //    oled_putstr("VCO: ");
    //    oled_putstr(wave_name[vco_wave]);
    //}

    //  VCO Chorus
    //if(vco_chorus_flg == -1){
    //    vco_chorus_flg = vco_chorus;  // vco_chorus value = 0 or 1
    //    oled_locate(0,2);
    //    oled_putstr("VCO: Chorus ");
    //    oled_putstr(chorus_name[vco_chorus]);
    //}

    //  LFO Wave
    //if(lfo_wave_select_flg == -1){
    //    lfo_wave_select_flg = KEY_OFF;
    //    oled_locate(0,3);
    //    oled_putstr("LFO: ");
    //    oled_putstr(lfo_wave_name[lfo_wave]);
    //}


    // *** controlle by key switch  ************
    // EG1 envelope pattern
    //  select EG1 Pattern type
    if(key_switch[ID_EG1_AR] == KEY_ON){
        if(eg_ptrn[EG1] != EG_AR){
            HAL_Delay(10);
            eg_ptrn[EG1] = EG_AR; // AR
        }
    }else if(key_switch[ID_EG1_ASR] == KEY_ON){
        if(eg_ptrn[EG1] != EG_ASR){
            HAL_Delay(10);
            eg_ptrn[EG1] = EG_ASR;     // ASR
        }
    }else{
        eg_ptrn[EG1] = EG_KEYOFF;
    }

    // EG2 envelope pattern
    //  select EG2 Pattern type
    if(key_switch[ID_EG2_AR] == KEY_ON){
        if(eg_ptrn[EG2] != EG_AR){
            HAL_Delay(10);
            eg_ptrn[EG2] = EG_AR; // AR
        }
    }else if(key_switch[ID_EG2_ASR] == KEY_ON){
        if(eg_ptrn[EG2] != EG_ASR){
            HAL_Delay(10);
            eg_ptrn[EG2] = EG_ASR;     // ASR
        }
    }else{
        eg_ptrn[EG2] = EG_KEYOFF;
    }

    //  select EG LFO Trigger
    if(key_switch[ID_EG_TRIG_LFO1] == KEY_ON){
        if(lfo1_trig == 0){
            HAL_Delay(10);
            lfo1_trig = 1;   // lfo1 trigger on
        }
    }else if(key_switch[ID_EG_TRIG_LFO2] == KEY_ON){
        if(lfo2_trig == 0){
            HAL_Delay(10);
            lfo2_trig = 1;   // lfo2 trigger on
        }
    }else{
        if(lfo1_trig || lfo2_trig){
            midi_voice_release(rpt_note->numb);
            rpt_note = NULL;
        }
        lfo1_trig = 0;   // lfo1 trigger off
        lfo2_trig = 0;   // lfo1 trigger on
    }


    // select VCO wave
    if(key_switch[ID_VCO_WAVE] == KEY_ON){
        HAL_Delay(10);
        if(key_switch[ID_VCO_WAVE] == KEY_ON && vco_wave_select_flg == KEY_OFF){
            vco_wave_select_flg = KEY_ON;
            vco_wave++;
            vco_wave %= 4;
            display_current_setting();
            //oled_locate(0,1);
            //oled_putstr("VCO: ");
            //oled_putstr(wave_name[vco_wave]);
        }
    }else{
        vco_wave_select_flg = KEY_OFF;
    }

    //  select LFO wave
    if(key_switch[ID_LFO_WAVE] == KEY_ON){
        HAL_Delay(10);
        if(key_switch[ID_LFO_WAVE] == KEY_ON && lfo_wave_select_flg == KEY_OFF){
            lfo_wave_select_flg = KEY_ON;
            lfo_wave++;
            lfo_wave %= 3;
            lfo_wave_table = lfo_wave_select[lfo_wave];
            display_current_setting();
            //oled_locate(0,3);
            //oled_putstr("LFO: ");
            //oled_putstr(lfo_wave_name[lfo_wave]);
        }
    }else{
        lfo_wave_select_flg = KEY_OFF;
    }

    //  select AMP controlle (EG1/EG2 or KBD)
    if(key_switch[ID_AMP_CTL_EG1] == KEY_ON){
        if(amp_ctl != AMP_CTL_EG1){
            HAL_Delay(10);
            amp_ctl = AMP_CTL_EG1;
        }
    }else if(key_switch[ID_AMP_CTL_EG2] == KEY_ON){
        if(amp_ctl != AMP_CTL_EG2){
            HAL_Delay(10);
            amp_ctl = AMP_CTL_EG2;
        }
    }else{
        amp_ctl = AMP_CTL_KBD;
    }

    //  VCO chorus
    if(vco_chorus_flg != vco_chorus){   // vco_chorus is 1 or 0
        vco_chorus_flg = vco_chorus;    //   vco_chorus ctrl by potentio control
        display_current_setting();
        //oled_locate(0,2);
        //oled_putstr("VCO: Chorus ");
        //oled_putstr(chorus_name[vco_chorus]);
    }

    // VCO Echo
    if(key_switch[ID_VCO_ECHO] == 0){
        echo_time = 1; // echo on
    }else{
        echo_time = 0; // echo off
    }

    //*** Control by REC/PLAY menu
    read_menu_key();
    if(key_switch[ID_MENU_SET] == KEY_ON ||
       key_switch[ID_MENU_UP] == KEY_ON ||
       key_switch[ID_MENU_DOWN] == KEY_ON) {
        do {
            read_menu_key();
        }while(key_switch[ID_MENU_SET] == KEY_ON ||
                key_switch[ID_MENU_UP] == KEY_ON ||
                key_switch[ID_MENU_DOWN] == KEY_ON);

        main_menu();
        display_current_setting();
        HAL_Delay(500);
    }
}

/*===============================================================
     read key switch
===============================================================*/

void read_key_switch(void)
{
    key_switch[0] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_10);
    key_switch[1] = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_8);
    key_switch[2] = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_9);
    key_switch[3] = HAL_GPIO_ReadPin(GPIOC, GPIO_PIN_7);
    key_switch[4] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_6);
    key_switch[5] = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_7);
    key_switch[6] = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_6);
    key_switch[7] = HAL_GPIO_ReadPin(GPIOA, GPIO_PIN_10);
    key_switch[8] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_3);
    key_switch[9] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_5);
    key_switch[10] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_4);
  //  key_switch[11] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_13);
  //  key_switch[12] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_14);
  //  key_switch[13] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_15);
}

void read_menu_key(void)
{
    // menu key
    key_switch[11] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_13);
    key_switch[12] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_14);
    key_switch[13] = HAL_GPIO_ReadPin(GPIOB, GPIO_PIN_15);

}

void dump_key(void)
{
//    int n;
//    for(n=0; n<14; n++){
//        UART2_putstring(" key["); UART2_putdec(n); UART2_putstring("]=");
//        UART2_putdec(key_switch[n]);// UART2_putcrlf();
//    }
//    UART2_putcrlf();
    int n;
    static char *kstr[2]={ "on ", "off", };
    for(n=0; n<14; n+=2){
        oled_locate(0, n/2);
        sprintf(pbuf, "%2d=%3s, %2d=%3s  ", n, kstr[key_switch[n]], n+1, kstr[key_switch[n+1]]);
        oled_putstr(pbuf);
    }
}

/*** end of file ***********************************************/


