/***********************************************************************
 * menu.c
 *
 *  Created on: Nov 1, 2024
 *      Author: sei_s
 **********************************************************************/

#include <stdio.h>
#include <string.h>
#include "main.h"
#include "flash.h"
#include "uart2.h"
#include "graphic.h"
#include "oled_128x64.h"
#include "synth.h"
#include "eg.h"
#include "osc.h"
#include "lfo.h"
#include "filter.h"
#include "amp.h"
#include "echo.h"
#include "analog_synth.h"
#include "sd_file.h"
#include "key.h"
#include "menu.h"


/*=======================================================================
    definition
========================================================================*/


/*=======================================================================
    global variable
========================================================================*/


/*=======================================================================
    function prototype
========================================================================*/


void display_current_setting(void);
void main_menu(void);
int  get_menu_key(void);

static void display_main_menu(int select);
static void display_midi_channel(int chn);

static void midi_channel_mute(void);
static void save_setting(void);
static void load_setting(void);
static void change_play_mode(void);
static void recording(void);
static void playing(void);
static void dump_pot_key(void);


/*===============================================================
     display current setting
===============================================================*/

void display_current_setting(void)
{
    static char *wave_name[] =   { "FM         ", "Saw        ", "Pulse      ", "Noise      ", };
    static char *lfo_wave_name[]={ "Sine       ", "Saw        ", "Pulse      ", };
    static char *chorus_name[] = { "Off ",        "on  ", };


    oled_locate(0,0);
    oled_putstr("= Synthesizer  =");

    oled_locate(0,1);
    oled_putstr("OSC: "); oled_putstr(wave_name[vco_wave]);

    oled_locate(0,2);
    oled_putstr("OSC: Chorus ");
    oled_putstr(chorus_name[vco_chorus]);

    oled_locate(0,3);
    oled_putstr("LFO: ");
    oled_putstr(lfo_wave_name[lfo_wave]);

    oled_locate(0,4);
    oled_putstr("                ");

    oled_locate(0,5);
    oled_putstr("                ");

    oled_locate(0,6);
    oled_putstr(play_mode == NORMAL_MODE? "PLAY mode: NORML":"PLAY mode: FIXED");

    oled_locate(0,7);
    oled_putstr(UsbMode==USB_HOST_MODE ? "USB mode: HOST  ":"USB mode: DEVICE");

}


/*=======================================================================
    main menu
========================================================================*/

static char *main_menu_str[]= {
      // 1234567890123456
        "=== MainMenu ===",  // 0
        "Midi Chanel Mute",  // 1
        "Save Setting    ",  // 2
        "Load Setting    ",  // 3
        "Play normal mode",  // 4
        "Recording       ",  // 5
        "Playing         ",  // 6
        "Dump Pot/Key    ",  // 7
};

/*-----------------------------------------------------------------------
    display main menu
-----------------------------------------------------------------------*/

static void display_main_menu(int select)
{
    int n;
    char play_mode_str[20];

    sprintf(play_mode_str, play_mode == NORMAL_MODE? "play normal mode":"play fixed mode ");
    main_menu_str[4] = play_mode_str;

    for(n=0; n<8; n++){
       oled_locate(0, n);
       if(n==select){
            oled_putstr_reverse(main_menu_str[n]);
        }else{
            oled_putstr(main_menu_str[n]);
        }
    }
}


/*-----------------------------------------------------------------------
    main menu function
-----------------------------------------------------------------------*/

void main_menu(void)
{
    int  select;
    int  key;

    // now recording ?
    if(recording_flg == 1){
        stop_recording();  // stop recordig
    }

    if(playing_flg == 1){
        stop_playing();    // stop playing
    }

    select = 1;   // select valid value  1 to 7

    for(;;){
        display_main_menu(select);

        key = get_menu_key();
        switch(key){
        case SET_KEY_ON:
            if(select == 1){
                midi_channel_mute();
                return;
            }else if(select == 2){
                save_setting();
            }else if(select == 3){
                load_setting();
            }else if(select == 4){
                change_play_mode();
                return;
            }else if(select == 5){
                recording();
                if(recording_flg == 1){
                    return;
                }
            }else if(select == 6){
                playing();
                if(playing_flg == 1){
                    return;
                }
            }else if(select == 7){
                dump_pot_key();
                return;
            }
            break;

        case UP_KEY_ON:
            select--;
            if(select < 1) select = 7;
            break;

        case DOWN_KEY_ON:
            select++;
            if(select > 7) select = 1;
            break;

        case END_KEY:
            return;

        }
    }
}


/*=======================================================================
    midi channel mute
========================================================================*/

static char midi_mute_str[8][20];
/*
      // 1234567890123456
        " 1:off   2:off  ",  // 0
        " 3:off   4:off  ",  // 1
        " 5:on    6:on   ",  // 2
        " 7:on    8:on   ",  // 3
        " 9:on   10:on   ",  // 4
        "11:on   12:on   ",  // 5
        "13:on   14:on   ",  // 6
        "15:off  16:off  ",  // 7
*/

/*-----------------------------------------------------------------------
     display midi channel list
-----------------------------------------------------------------------*/

static void display_midi_channel(int chn)
{
    int n;
    int c;

    for(n=0; n<8; n++){
        oled_locate(0, n);
        if(chn == (n*2 +1)){ // left reverse
            c = midi_mute_str[n][8];
            midi_mute_str[n][8]=0;
            oled_putstr_reverse(midi_mute_str[n]);
            midi_mute_str[n][8]=c;
            oled_putstr(&(midi_mute_str[n][8]));  // right

        }else if(chn == (n*2 +2)){ // right reverse
            c = midi_mute_str[n][8];
            midi_mute_str[n][8]=0;
            oled_putstr(midi_mute_str[n]);
            midi_mute_str[n][8]=c;
            oled_putstr_reverse(&(midi_mute_str[n][8]));  // right

        }else{
            oled_putstr(midi_mute_str[n]);
        }
    }
}

/*-----------------------------------------------------------------------
    void midi_channel_mute(void)
-----------------------------------------------------------------------*/

static void midi_channel_mute(void)
{
    int n;
    int mute_chn;
    int key;

    for(n=0;n<8;n++){
        sprintf(midi_mute_str[n], "%2d:%3s  %2d:%3s  ",
                     n*2+1, (play_ch[n*2] ? "on": "off"),
                     n*2+2, (play_ch[n*2+1] ? "on": "off") );
    }

    mute_chn = 1;
    for(;;){
        display_midi_channel(mute_chn);

        key = get_menu_key();
        switch(key){
        case SET_KEY_ON:
            play_ch[mute_chn-1] ^= 1;  // toggle
            n=(mute_chn-1)/2;
            sprintf(midi_mute_str[n], "%2d:%3s  %2d:%3s  ",
                    n*2+1, (play_ch[n*2] ? "on": "off"),
                    n*2+2, (play_ch[n*2+1] ? "on": "off") );
            break;

        case UP_KEY_ON:
            mute_chn--;
            if(mute_chn < 1){
                mute_chn = 16;
            }
            break;

        case DOWN_KEY_ON:
            mute_chn++;
            if(mute_chn > 16){
                mute_chn = 1;
            }
            break;

        case END_KEY:
            return;  // return to main menu
        }
    }
}



/*=======================================================================
   save potentio and keys setting value
========================================================================*/

static void save_setting(void)
{
    int  numb;
    int  key;
    char buf[6];

    numb = 1;
    for(;;){
        oled_locate(0, 2);
        oled_putstr("save number=  ");
        sprintf(buf, "%2d", numb);
        oled_putstr_reverse(buf);

        key = get_menu_key();
        switch(key){
        case SET_KEY_ON:
            save_parameter(numb);
            return;

        case UP_KEY_ON:
            numb++;
            if(numb >PARAM_MAX) numb=1;
            break;

        case DOWN_KEY_ON:
            numb--;
            if(numb <1) numb=PARAM_MAX;
            break;

        case END_KEY:
            return;  // return to main menu
        }
    }
}


/*=======================================================================
   load potentio and keys setting value from flash
========================================================================*/

static void load_setting(void)
{
    int  numb;
    int  key;
    char buf[6];

    numb = 1;
    for(;;){
        oled_locate(0, 3);
        oled_putstr("Load number=  ");
        sprintf(buf, "%2d", numb);
        oled_putstr_reverse(buf);

        key = get_menu_key();
        switch(key){
        case SET_KEY_ON:
            if(load_parameter(numb) != -1){ // not fail
                play_mode = FIXED_MODE;
            }
            return;

        case UP_KEY_ON:
            numb++;
            if(numb >PARAM_MAX) numb=1;
            break;

        case DOWN_KEY_ON:
            numb--;
            if(numb <1) numb=PARAM_MAX;
            break;

        case END_KEY:
            return;  // return to main menu
        }
    }
}


/*=======================================================================
   change play mode
========================================================================*/

static void change_play_mode(void)
{
    if(play_mode == NORMAL_MODE){
        play_mode = FIXED_MODE;
    }else{
        play_mode = NORMAL_MODE;
    }
}


/*=======================================================================
   recording to SD Card
========================================================================*/

static void recording(void)
{
    int  numb;
    int  key;
    char buf[6];

    numb = 1;
    for(;;){
        oled_locate(0, 5);
        oled_putstr("record number=");
        sprintf(buf, "%2d", numb);
        oled_putstr_reverse(buf);

        key = get_menu_key();
        switch(key){
        case SET_KEY_ON:
            start_recording(numb-1);
            return;

        case UP_KEY_ON:
            numb++;
            if(numb >MAX_FILES) numb=1;
            break;

        case DOWN_KEY_ON:
            numb--;
            if(numb <1) numb=MAX_FILES;
            break;

        case END_KEY:
            return;  // return to main menu
        }
    }
}


/*=======================================================================
   playing from SD Card
========================================================================*/

static void playing(void)
{
    int  numb;
     int  key;
     char buf[6];

     numb = 1;
     for(;;){
         oled_locate(0, 6);
         oled_putstr("play number=  ");
         sprintf(buf, "%2d", numb);
         oled_putstr_reverse(buf);

         key = get_menu_key();
         switch(key){
         case SET_KEY_ON:
             start_playing(numb-1);
             return;

         case UP_KEY_ON:
             numb++;
             if(numb >MAX_FILES) numb=1;
             break;

         case DOWN_KEY_ON:
             numb--;
             if(numb <1) numb=MAX_FILES;
             break;

         case END_KEY:
             return;  // return to main menu
         }
     }
}


/*=======================================================================
   display  potentio data or key data
========================================================================*/

static void dump_pot_key(void)
{
    if(dump_pot_flg == 0 && dump_key_flg == 0){
        dump_pot_flg = 1;
    }else if(dump_pot_flg){
        dump_pot_flg = 0;
        dump_key_flg = 1;
    }else if(dump_key_flg){
        dump_key_flg = 0;
    }
}

/*=======================================================================
    get menu key
========================================================================*/

int get_menu_key(void)
{
    static int prev_set_key = KEY_OFF;
    static int prev_up_key = KEY_OFF;
    static int prev_down_key = KEY_OFF;

    read_menu_key();

    if(prev_set_key != key_switch[ID_MENU_SET]){
        HAL_Delay(10);
        if(prev_set_key != key_switch[ID_MENU_SET]){
            prev_set_key = key_switch[ID_MENU_SET];
            if(key_switch[ID_MENU_SET] == KEY_ON){
                if(prev_up_key == KEY_ON || prev_down_key == KEY_ON){
                    return END_KEY;
                }
                return SET_KEY_ON;

            }else{
                return SET_KEY_OFF;
            }
        }
    }

    if(prev_up_key != key_switch[ID_MENU_UP]){
        HAL_Delay(10);
        if(prev_up_key != key_switch[ID_MENU_UP]){
            prev_up_key = key_switch[ID_MENU_UP];
            if(key_switch[ID_MENU_UP] == KEY_ON){
                if(prev_set_key == KEY_ON || prev_down_key == KEY_ON){
                    return END_KEY;
                }
                return UP_KEY_ON;
            }else{
                return UP_KEY_OFF;
            }
        }
    }

    if(prev_down_key != key_switch[ID_MENU_DOWN]){
        HAL_Delay(10);
        if(prev_down_key != key_switch[ID_MENU_DOWN]){
            prev_down_key = key_switch[ID_MENU_DOWN];
            if(key_switch[ID_MENU_DOWN] == KEY_ON){
                if(prev_set_key == KEY_ON || prev_up_key == KEY_ON){
                    return END_KEY;
                }
                return DOWN_KEY_ON;
            }else{
                return DOWN_KEY_OFF;
            }
        }
    }

    return NO_KEY;

}


/*** end of file ********************************************************/
