/***************************************************************
 * osc.c
 *
 *  Created on: Aug 22, 2024
 *      Author: sei_s
 **************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "synth.h"
#include "echo.h"
#include "osc.h"
#include "eg.h"
#include "lfo.h"
#include "uart2.h"


/*===============================================================
     private variable
===============================================================*/


/*===============================================================
     global variable
===============================================================*/

//
//  wave data table
//
#include "wave_table.h"
//float     sine_wave_table[1024];
//float     noise_table[1024];
//float     saw_wave_table[5][1024];
//float     pulse_wave_table[5][1024];

float     use_wave_table[] = { 200.0F, 400.0F, 800.0F, 1600.0F, 4000.0F, };


//
// wave select
//
int                vco_wave;

//
// FM modulation
//
volatile float     fm_ratio;
volatile float     fm_depth;

//
// PORTAMENT
//
float              vco_prev_delta;
int                vco_prev_note;
volatile int       vco_portament;
volatile int       vco_portament1;


//
//  pitch control
//
volatile float     vco_lfo1_mod_depth;
volatile float     vco_lfo2_mod_depth;
volatile float     vco_eg1_mod_depth;
volatile float     vco_eg2_mod_depth;

volatile float     vco_pitch;          // +/- 1octave
volatile float     vco_pitch_shift;

//
// amplitude control
//
volatile float     vco_lfo1_tremoro_depth;
volatile float     vco_lfo2_tremoro_depth;

//
//  efect control
//
volatile int       vco_chorus;

//
//  private
//  use make wave table
//static int   harmonics[]={ 100, 50, 25, 12, 5, };


/*===============================================================
    function prototype
===============================================================*/

/*===============================================================
    osc initialize
===============================================================*/

void osc_init(void)
{
	vco_wave = SAW_WAVE;
	vco_prev_delta = 0.0F;;
	vco_prev_note = 0;
	vco_portament = 1;  // min=1, max=500;
	vco_pitch = 1.0F;   // pitch tuning
    vco_chorus = 0;
    fm_ratio = 0.0F;
    fm_depth = 0.0F;    // 0.0 - 5.0f

    fm_depth = 1.0F;

#if 0
	//
	// make wave table program
	//
	char  pbuf[128];
    int   n, m;
	int   table;
	float sum;

	// make sine_wave_table
	// sine valu -0.25 to 0.25
	for(n=0; n<1024; n++){
		sine_wave_table[n]= 0.25F*sinf(2.0F*PI*(float)n/1024.0F);
	}


	UART2_putstring("const float sine_wave_table[] = {\r\n    ");
	for(n=0; n<1024; n++){
		sprintf(pbuf, "%.8f, ", sine_wave_table[n]);
        UART2_putstring(pbuf);
        if((n%8)==7) UART2_putstring("\r\n    ");
	}
	UART2_putstring("};\r\n\r\n");

	// make saw_wave_table
	for(table=0; table<5; table++){
		for(n=0; n<1024; n++){
			sum = 0.0F;
			for(m=1;m<=harmonics[table];m++){
				sum += (2.0F/PI)*sinf(2.0F*PI*(float)n*(float)m/1024.0F)/(float)m;
			}
			saw_wave_table[table][n]=0.25F*sum;
		}
	}

	UART2_putstring("const float saw_wave_table[][] = {\r\n");
	for(table=0; table<5; table++){
    	UART2_putstring(" {\r\n   ");
	    for(n=0; n<1024; n++){
		    sprintf(pbuf, "%.8f, ", saw_wave_table[table][n]);
            UART2_putstring(pbuf);
            if((n%8)==7) UART2_putstring("\r\n   ");
	    }
    	UART2_putstring("},\r\n");
	}
	UART2_putstring("};\r\n\r\n");


	// make pulse_wave_table
	for(table=0; table<5; table++){
		for(n=0; n<1024; n++){
			sum = 0.0F;
			for(m=1;m<harmonics[table];m+=2){
				sum += (2.0F/PI)*sinf(2.0F*PI*(float)n*(float)m/1024.0F)/(float)m;
			}
			pulse_wave_table[table][n]=0.25F*sum;
		}
	}

	UART2_putstring("const float pulse_wave_table[][] = {\r\n");
	for(table=0; table<5; table++){
    	UART2_putstring(" {\r\n   ");
	    for(n=0; n<1024; n++){
		    sprintf(pbuf, "%.8f, ", pulse_wave_table[table][n]);
            UART2_putstring(pbuf);
            if((n%8)==7) UART2_putstring("\r\n   ");
	    }
    	UART2_putstring("},\r\n");
	}
	UART2_putstring("};\r\n\r\n");

  #if 0
	// sine wave display
	int i;
	UART2_putstring("sine wave\r\n");
	grp_clear();
	for(i=0;i<128;i++){
		UART2_putdec(sine_wave_table[i*8]*110.0F+32.0F); UART2_putchar(' ');
		if((i%32)==31)UART2_putcrlf();
		grp_pset(i, sine_wave_table[i*8]*110.0F+32.0F, GRP_PON);
	}
	UART2_putcrlf();
	oled_graphic();

	UART2_putstring("next saw wave, hit any key!\r\n");
	UART2_getchar();

	for(table=0; table <5;table++){
        UART2_putstring("saw_wave max harmonics="); UART2_putdec(harmonics[table]);UART2_putcrlf();
		grp_clear();
		for(i=0;i<128;i++){
			UART2_putdec(saw_wave_table[table][i*8]*110.0F+32.0F); UART2_putchar(' ');
			if((i%32)==31)UART2_putcrlf();
			grp_pset(i, saw_wave_table[table][i*8]*110.0F+32.0F, GRP_PON);
		}
		UART2_putcrlf();
		oled_graphic();

		UART2_putstring("next wave, hit any key!\r\n");
		UART2_getchar();
	}

	for(table=0; table <5;table++){
        UART2_putstring("pulse_wave max harmonics="); UART2_putdec(harmonics[table]);UART2_putcrlf();
		grp_clear();
		for(i=0;i<128;i++){
			UART2_putdec(pulse_wave_table[table][i*8]*110.0F+32.0F); UART2_putchar(' ');
			if((i%32)==31)UART2_putcrlf();
			grp_pset(i, pulse_wave_table[table][i*8]*110.0F+32.0F, GRP_PON);
		}
		UART2_putcrlf();
		oled_graphic();

		UART2_putstring("next wave, hit any key!\r\n");
		UART2_getchar();
	}
  #endif
#endif

}


/*===============================================================
    osc process
===============================================================*/

inline float osc_process(MIDI_NOTE *note)
{
  float phase_1024;
  float fm_theta;
  int   curr_index;
  //int   next_index;
  //float curr_sample;
  //float next_sample;
  //float next_weight;
  //float curr_weight;

  float val;

  // white noise
  if(vco_wave == NOISE_WAVE){
	  val = ((float)random()/(float)RAND_MAX -0.5F)/2.0F;
	  goto tremoro;
	  // return val;
  }


  // portament
  if((note->sampl_cnt % vco_portament1) == 0){
      if(note->portament){
          note->delta *= note->portament_ratio;   // ratio=1.029302236  // up
          note->portament--;
      }                                           // ratio=0.971531941  // down
  }
  note->sampl_cnt++;

  //*** VCO modulation *****************
  //   Pitch tune
  //   LFO modulation (can use LFO #1 or LFO #2)
  //   EG modulation (can use EG1 or EG2)

  // case vibrate by LFO and EG
  // pitch_shift for chrous
  if(note->pitch_shift == 1){ // pitch plus shift
      note->theta += note->delta * vco_pitch * vco_pitch_shift *
              (1.0F + lfo1_level*vco_lfo1_mod_depth + note->eg[EG1].level * vco_eg1_mod_depth +
                      lfo2_level*vco_lfo2_mod_depth + note->eg[EG2].level * vco_eg2_mod_depth);

  }else if(note->pitch_shift == -1){  // pitch minus shift
      note->theta += note->delta * vco_pitch * (1.0F / vco_pitch_shift) *
              (1.0F + lfo1_level*vco_lfo1_mod_depth + note->eg[EG1].level * vco_eg1_mod_depth +
                      lfo2_level*vco_lfo2_mod_depth + note->eg[EG2].level * vco_eg2_mod_depth);

  }else{ // == 0, chorus off or base voice
      note->theta += note->delta * vco_pitch *
              (1.0F + lfo1_level*vco_lfo1_mod_depth + note->eg[EG1].level * vco_eg1_mod_depth +
                      lfo2_level*vco_lfo2_mod_depth + note->eg[EG2].level * vco_eg2_mod_depth);

  }

  //*** FM **********************************************
  if(vco_wave == SINE_WAVE){ // FM modulation
      fm_theta = note->theta * fm_ratio;    // fm_ratio = 0.0f - 20.0f
      fm_theta = fm_theta - floorf(fm_theta);
      phase_1024 = fm_theta * 1024.0F;
      curr_index = (int)floorf(phase_1024) & 0x3FF;

      fm_theta = note->theta + fm_depth * note->wave_table[curr_index];
      fm_theta = fm_theta - floorf(fm_theta);
      phase_1024 = fm_theta * 1024.0F;
      curr_index = (int)floorf(phase_1024) & 0x3FF;
      val = note->wave_table[curr_index];

      note->theta = note->theta - floorf(note->theta);
      goto tremoro;
  }

  //*** voice sampling *****************
  note->theta = note->theta - floorf(note->theta);
  phase_1024 = note->theta * 1024.0F;
  curr_index = (int)floorf(phase_1024) & 0x3FF;
  //next_index = (curr_index +1) & 0x000003FF;
  //curr_sample = note->wave_table[curr_index];
  //next_sample = note->wave_table[next_index];
  //next_weight = phase_1024 - floorf(phase_1024);
  //curr_weight = 1.0F - next_weight;
  //val = (curr_sample*curr_weight)+(next_sample*next_weight);

  val = note->wave_table[curr_index];

tremoro:
  //*** tremoro by lfo #2 ****************************
  val *= (1.0F + (lfo1_level*vco_lfo1_tremoro_depth +
                  lfo2_level*vco_lfo2_tremoro_depth )*5.0F);

  return val;
}


/*** end of file ***********************************************/


