/*
 * potentio.c
 *
 *  Created on: Aug 28, 2024
 *      Author: sei_s
 */

#include <stdio.h>
#include <string.h>
#include "main.h"
#include "synth.h"
#include "uart2.h"
#include "eg.h"
#include "osc.h"
#include "lfo.h"
#include "filter.h"
#include "amp.h"
#include "analog_synth.h"
#include "potentio.h"
#include "oled_128x64.h"


/*===============================================================
     global variable
===============================================================*/

extern ADC_HandleTypeDef hadc1;

POTENTIO pot[20] = {0};     // 0-4095  --> 0.0-0.9999
static int adchn[5]={ 0, 1, 8, 11, 10, };

int pot_read_flg;


/*===============================================================
     function prototype
===============================================================*/

void potentio_control(void);
void read_potentio(void);

static int  read_ADC(int ch);
static void ADC_Start(ADC_HandleTypeDef *hadc);
static void ADC_Stop(ADC_HandleTypeDef *hadc);
static HAL_StatusTypeDef ADC_PollForConversion(ADC_HandleTypeDef *hadc);


/*=======================================================================
    potentio control

  This function is called by Timer2 interrupt. 20msec interval.
========================================================================*/

void potentio_control(void)
{
    //HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);

    //*** read potentiometer ********************************
    if(play_mode == NORMAL_MODE){
        read_potentio();
        pot_read_flg = 1;
    }

    //*** VCO control ***************************************
    // Tune   0.5 - 2.0  (-1octave - +1octave)
    if(pot[ID_VCO_TUNE].val >= 2198){
        vco_pitch = 1.0F+(float)(pot[ID_VCO_TUNE].val -2198)/(1898.0F);
    }else if(pot[ID_VCO_TUNE].val <= 1898){
        vco_pitch = 0.5F + (float)(pot[ID_VCO_TUNE].val)/3796.0F;
    }else{
        vco_pitch = 1.0F;
    }
    //  Portament
    vco_portament = pot[ID_VCO_PORTAMENT].val * 600 / 4096;
    if(vco_portament == 0) vco_portament = 1;

    //  TREMORO
    if(pot[ID_VCO_TREMORO].val >= 2198){
        vco_lfo2_tremoro_depth = (float)(pot[ID_VCO_TREMORO].val -2198)/(1898.0F);
        vco_lfo1_tremoro_depth = 0.0F;
    }else if(pot[ID_VCO_TREMORO].val <= 1898){
        vco_lfo1_tremoro_depth = (float)(1898- pot[ID_VCO_TREMORO].val)/1898.0F;
        vco_lfo2_tremoro_depth = 0.0F;
    }else{
        vco_lfo1_tremoro_depth = 0.0F;
        vco_lfo2_tremoro_depth = 0.0F;
    }


    //  LFO modulation
    if(pot[ID_VCO_LFO_MOD].val >= 2198){
        vco_lfo2_mod_depth = (float)(pot[ID_VCO_LFO_MOD].val -2198)/(1898.0F);
        vco_lfo1_mod_depth = 0.0F;
    }else if(pot[ID_VCO_LFO_MOD].val <= 1898){
        vco_lfo1_mod_depth = (float)(1898-pot[ID_VCO_LFO_MOD].val)/(1898.0F);
        vco_lfo2_mod_depth = 0.0F;
    }else{
        vco_lfo1_mod_depth = 0.0F;
        vco_lfo2_mod_depth = 0.0F;
    }

    //  EG modulation
    //     0-1.0
    if(pot[ID_VCO_EG_MOD].val >= 2198){
        vco_eg2_mod_depth = (float)(pot[ID_VCO_EG_MOD].val -2198)/(1898.0F);
        vco_eg1_mod_depth = 0.0F;
    }else if(pot[ID_VCO_EG_MOD].val <= 1898){
        vco_eg1_mod_depth = (float)(1898- pot[ID_VCO_EG_MOD].val)/1898.0F;
        vco_eg2_mod_depth = 0.0F;
    }else{
        vco_eg1_mod_depth = 0.0F;
        vco_eg2_mod_depth = 0.0F;
    }

    //  Chorus
    if(pot[ID_VCO_CHORUS].val <= 200){
        vco_chorus = 0;
    }else{
        vco_chorus = 1;
        vco_pitch_shift = 1.0F + (float)(pot[ID_VCO_CHORUS].val -200)*0.01F/3896.0F;
    }

    //*** EG control *************************************
    // ATACK time ratio
    eg_atack_ratio[EG1] = 0.997f + (float)(pot[ID_EG1_ATACK].val)/1400000.0f;  // 0.0 < 1.0
    eg_atack_ratio[EG2] = 0.997f + (float)(pot[ID_EG2_ATACK].val)/1400000.0f;  // 0.0 < 1.0

    // DECAY time ration
    //eg_decay_ratio[EG1] = 0.995f + (float)(pot[ID_EG1_DECAY].val)/820000.0f;  // 0.0 < 1.0
    //eg_decay_ratio[EG2] = 0.995f + (float)(pot[ID_EG2_DECAY].val)/820000.0f;  // 0.0 < 1.0

    // RELEASE time ration
    eg_release_ratio[EG1] = 0.997f + (float)(pot[ID_EG1_RELEASE].val)/1400000.0f;  // 0.0 < 1.0
    eg_release_ratio[EG2] = 0.997f + (float)(pot[ID_EG2_RELEASE].val)/1400000.0f;  // 0.0 < 1.0

    // SUSTAIN level
    //eg_sustain_level[EG1] = (float)(pot[ID_EG1_SUSTAIN].val)/4096.0f;  // 0.0 < 1.0
    //eg_sustain_level[EG2] = (float)(pot[ID_EG2_SUSTAIN].val)/4096.0f;  // 0.0 < 1.0


    //*** LFO control *************************************
    //  LFO #1 freq
    lfo1_freq = (float)pot[ID_LFO1_FREQ].val * LFO_MAX_FREQ / 4096.0F;
    //lfo1_delta =  lfo1_freq / 40000.0F;
    lfo1_delta =  lfo1_freq * 1024.0F / 40000.0F;
    if(lfo1_freq < 0.01f){
        lfo1_theta = 0;
    }

    //  LFO #2 freq
    lfo2_freq = (float)pot[ID_LFO2_FREQ].val * LFO_MAX_FREQ / 4096.0F;
    //lfo2_delta =  lfo2_freq / 40000.0F;
    lfo2_delta =  lfo2_freq * 1024.0F / 40000.0F;
    if(lfo2_freq < 0.01f){
        lfo2_theta = 0;
    }


    //*** LPF control **************************************
    // cut off frequency
    lpf_cut = pot[ID_LPF_FREQ].val >> 5;  // 4095/32=127  0-127
    // resonance
    lpf_res = pot[ID_LPF_RES].val >> 9;   // 4095/512=7 0-7
    // KBD modulation
    vcf_kbd_mod_depth  = (float)(pot[ID_LPF_KBD_MOD].val)/4096.0F;

    // LFO modulation
    if(pot[ID_LPF_LFO_MOD].val >= 2198){
        vcf_lfo2_mod_depth = (float)(pot[ID_LPF_LFO_MOD].val -2198)/(1898.0F);
        vcf_lfo1_mod_depth = 0.0F;
    }else if(pot[ID_LPF_LFO_MOD].val <= 1898){
        vcf_lfo1_mod_depth = (float)(1898-pot[ID_LPF_LFO_MOD].val)/(1898.0F);
        vcf_lfo2_mod_depth = 0.0F;
    }else{
        vcf_lfo1_mod_depth = 0.0F;
        vcf_lfo2_mod_depth = 0.0F;
    }

    // EG modulation
    if(pot[ID_LPF_EG_MOD].val >= 2198){
        vcf_eg2_mod_depth = (float)(pot[ID_LPF_EG_MOD].val -2198)/(1898.0F);
        vcf_eg1_mod_depth = 0.0F;
    }else if(pot[ID_LPF_EG_MOD].val <= 1898){
        vcf_eg1_mod_depth = (float)(1898- pot[ID_LPF_EG_MOD].val)/1898.0F;
        vcf_eg2_mod_depth = 0.0F;
    }else{
        vcf_eg1_mod_depth = 0.0F;
        vcf_eg2_mod_depth = 0.0F;
    }



    //*** HPF control **************************************
    // cut off frequency
    hpf_cut = pot[ID_HPF_FREQ].val >> 5;  // 4095/32=127  0-127
    // resonance
    hpf_res = pot[ID_HPF_RES].val >> 9;   // 4095/512=7 0-7


    //*** ECHO control ***************************************
    //   ECHO time(delay)
    // echo_time = pot[ID_ECHO_TIME].val * MAX_ECHO_TIME / 4096;
    //   ECHO depth
    //echo_depth = (float)pot[ID_ECHO_DEPTH].val / 4096.0F;
    //
    // don't echo controlle. echo parameter is fixed. (see echo.c)

    //*** FM control *****************************************
    //   OPERATOR #1 frequency ratio
    fm_ratio = (float)pot[ID_FM_DEPTH].val * 12.0F / 4096.0F;  // 0.0 - 10.0F

    //HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET);

}


/*===============================================================
     dump potentio value
===============================================================*/

void dump_potentio(void)
{
//    int i;
//    for(i=0; i<20; i++){
//       sprintf(pbuf, "%2d: %4d, ", i+1, pot[i].val);
//       UART2_putstring(pbuf);
//       //UART2_putdec(pot[i].val); UART2_putchar(' ');
//       if((i%5)==4)UART2_putcrlf();
//    }
//    UART2_putcrlf();
//    UART2_putcrlf();

    int i;
    for(i=0; i<20; i+=3){
        oled_locate(0,i/3);
        if(i<18)sprintf(pbuf, "%04d, %04d, %04d", pot[i].val,  pot[i+1].val,  pot[i+2].val);
        else    sprintf(pbuf, "%04d, %04d,     ", pot[i].val,  pot[i+1].val);
        oled_putstr(pbuf);
    }

}


/*===============================================================
     read potentiometer
===============================================================*/

void read_potentio(void)
{
    int      sw, ch;
    int      adc_val;
    POTENTIO *p;

    //HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);

    for(sw=0; sw <4; sw++){
    	switch(sw){
    	case 0:
    		// switch 1 on
       	    HAL_GPIO_WritePin(GPIOA, GPIO_PIN_15, GPIO_PIN_RESET);
       	    break;
    	case 1:
    		// switch 2 on
       	    HAL_GPIO_WritePin(GPIOB, GPIO_PIN_7, GPIO_PIN_RESET);
       	    break;
    	case 2:
    		// switch 3 on
       	    HAL_GPIO_WritePin(GPIOC, GPIO_PIN_2, GPIO_PIN_RESET);
       	    break;
    	case 3:
    		// switch 4 on
       		HAL_GPIO_WritePin(GPIOC, GPIO_PIN_3, GPIO_PIN_RESET);
       	    break;
    	}

        for(ch=0; ch<5; ch++){
            adc_val = read_ADC(adchn[ch]);
        	p = &pot[ch*4+sw];
    		p->sum += adc_val;
    		p->sum -= p->prev[p->idx];
    		p->prev[p->idx] = adc_val;
    		p->idx++;
    		p->idx %= 16;
    		p->val = p->sum /16;
    	}
        HAL_GPIO_WritePin(GPIOC, GPIO_PIN_2|GPIO_PIN_3, GPIO_PIN_SET);
        HAL_GPIO_WritePin(GPIOA, GPIO_PIN_15, GPIO_PIN_SET);
        HAL_GPIO_WritePin(GPIOB, GPIO_PIN_7, GPIO_PIN_SET);
    }
    //HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_RESET);
}


static int read_ADC(int ch)
{
	int adc_val;

    ADC_ChannelConfTypeDef sConfig = {0};
    sConfig.Channel = ch;
    sConfig.Rank = 1;
    sConfig.SamplingTime = ADC_SAMPLETIME_84CYCLES;  // 56, 84,112, 144, 480
    if(HAL_ADC_ConfigChannel(&hadc1, &sConfig) != HAL_OK){
        UART2_putstring("HAL_ADC_ConfigChannel() error\r\n");
        return 0;
    }
    adc_val = 0;
    HAL_NVIC_DisableIRQ(TIM3_IRQn);
	//HAL_ADC_Start(&hadc1);
    ADC_Start(&hadc1);
//    if(HAL_ADC_PollForConversion(&hadc1, 1) == HAL_OK){
    if(ADC_PollForConversion(&hadc1) == HAL_OK){
        adc_val = HAL_ADC_GetValue(&hadc1);
    }else{
        UART2_putstring("ADC_PollForConversion() error\r\n");
    }
    //HAL_ADC_Stop(&hadc1);
    ADC_Stop(&hadc1);
    HAL_NVIC_EnableIRQ(TIM3_IRQn);
    return adc_val <= 40 ? 0:  adc_val;
}

static void ADC_Start(ADC_HandleTypeDef *hadc)
{
    //  __IO uint32_t counter = 0U;
    ADC_Common_TypeDef *tmpADC_Common;

    /* Enable the ADC peripheral */
    /* Check if ADC peripheral is disabled in order to enable it and wait during
    Tstab time the ADC's stabilization */
//    if((hadc->Instance->CR2 & ADC_CR2_ADON) != ADC_CR2_ADON){
        /* Enable the Peripheral */
        __HAL_ADC_ENABLE(hadc);
//    }

    /* Start conversion if ADC is effectively enabled */
    if(HAL_IS_BIT_SET(hadc->Instance->CR2, ADC_CR2_ADON)){
        /* Set ADC state                                                          */
        /* - Clear state bitfield related to regular group conversion results     */
        /* - Set state bitfield related to regular group operation                */
        ADC_STATE_CLR_SET(hadc->State,
                          HAL_ADC_STATE_READY | HAL_ADC_STATE_REG_EOC | HAL_ADC_STATE_REG_OVR,
                          HAL_ADC_STATE_REG_BUSY);

        /* If conversions on group regular are also triggering group injected,    */
        /* update ADC state.                                                      */
        if (READ_BIT(hadc->Instance->CR1, ADC_CR1_JAUTO) != RESET){
             ADC_STATE_CLR_SET(hadc->State, HAL_ADC_STATE_INJ_EOC, HAL_ADC_STATE_INJ_BUSY);
        }

        /* State machine update: Check if an injected conversion is ongoing */
        if (HAL_IS_BIT_SET(hadc->State, HAL_ADC_STATE_INJ_BUSY)){
             /* Reset ADC error code fields related to conversions on group regular */
             CLEAR_BIT(hadc->ErrorCode, (HAL_ADC_ERROR_OVR | HAL_ADC_ERROR_DMA));
        }else{
             /* Reset ADC all error code fields */
             ADC_CLEAR_ERRORCODE(hadc);
        }

        /* Pointer to the common control register to which is belonging hadc    */
        /* (Depending on STM32F4 product, there may be up to 3 ADCs and 1 common */
        /* control register)                                                    */
        tmpADC_Common = ADC_COMMON_REGISTER(hadc);

        /* Clear regular group conversion flag and overrun flag */
        /* (To ensure of no unknown state from potential previous ADC operations) */
        __HAL_ADC_CLEAR_FLAG(hadc, ADC_FLAG_EOC | ADC_FLAG_OVR);

        /* Check if Multimode enabled */
        if(HAL_IS_BIT_CLR(tmpADC_Common->CCR, ADC_CCR_MULTI)){
            /* if no external trigger present enable software conversion of regular channels */
            if((hadc->Instance->CR2 & ADC_CR2_EXTEN) == RESET){
              /* Enable the selected ADC software conversion for regular group */
              hadc->Instance->CR2 |= (uint32_t)ADC_CR2_SWSTART;
            }
        }else{
            /* if instance of handle correspond to ADC1 and  no external trigger present enable software conversion of regular channels */
            if((hadc->Instance == ADC1) && ((hadc->Instance->CR2 & ADC_CR2_EXTEN) == RESET)){
                /* Enable the selected ADC software conversion for regular group */
                 hadc->Instance->CR2 |= (uint32_t)ADC_CR2_SWSTART;
             }
        }
    }
}


static void  ADC_Stop(ADC_HandleTypeDef *hadc)
{
    /* Disable ADC peripheral */
    __HAL_ADC_DISABLE(hadc);

    /* Check if ADC is effectively disabled */
    if(HAL_IS_BIT_CLR(hadc->Instance->CR2, ADC_CR2_ADON)){
      /* Set ADC state */
      ADC_STATE_CLR_SET(hadc->State,
                        HAL_ADC_STATE_REG_BUSY | HAL_ADC_STATE_INJ_BUSY,
                        HAL_ADC_STATE_READY);
    }
}

static HAL_StatusTypeDef ADC_PollForConversion(ADC_HandleTypeDef *hadc)
{
//    if (HAL_IS_BIT_SET(hadc->Instance->CR2, ADC_CR2_EOCS) &&
//        HAL_IS_BIT_SET(hadc->Instance->CR2, ADC_CR2_DMA)    ){
//        /* Update ADC state machine to error */
//        SET_BIT(hadc->State, HAL_ADC_STATE_ERROR_CONFIG);
//        return HAL_ERROR;
//    }

   /* Check End of conversion flag */
    while(!(__HAL_ADC_GET_FLAG(hadc, ADC_FLAG_EOC))){
          ;
    }

    /* Clear regular group conversion flag */
     __HAL_ADC_CLEAR_FLAG(hadc, ADC_FLAG_STRT | ADC_FLAG_EOC);

      /* Update ADC state machine */
    SET_BIT(hadc->State, HAL_ADC_STATE_REG_EOC);

    /* Determine whether any further conversion upcoming on group regular       */
    /* by external trigger, continuous mode or scan sequence on going.          */
    /* Note: On STM32F4, there is no independent flag of end of sequence.       */
    /*       The test of scan sequence on going is done either with scan        */
    /*       sequence disabled or with end of conversion flag set to            */
    /*       of end of sequence.                                                */
    if(ADC_IS_SOFTWARE_START_REGULAR(hadc)                   &&
         (hadc->Init.ContinuousConvMode == DISABLE)            &&
         (HAL_IS_BIT_CLR(hadc->Instance->SQR1, ADC_SQR1_L) ||
          HAL_IS_BIT_CLR(hadc->Instance->CR2, ADC_CR2_EOCS)  )   ){
        /* Set ADC state */
        CLEAR_BIT(hadc->State, HAL_ADC_STATE_REG_BUSY);

        if (HAL_IS_BIT_CLR(hadc->State, HAL_ADC_STATE_INJ_BUSY)){
            SET_BIT(hadc->State, HAL_ADC_STATE_READY);
        }
    }
    return HAL_OK;
}


/*** end of file ***********************************************/
