/*
 * uart2.c
 *
 *  Created on: Jul 23, 2024
 *      Author: sei_s
 */


#include <string.h>
#include "main.h"
#include "uart2.h"

/*------------------------------------------------------------------------------
    definition
------------------------------------------------------------------------------*/

// UART2 Tx,Rx ring buffer
#define TXBUFSIZE   1024
#define RXBUFSIZE   256


/*------------------------------------------------------------------------------
    private variable
------------------------------------------------------------------------------*/

static char TxBuf[TXBUFSIZE];
static char RxBuf[RXBUFSIZE];

//static volatile char    rbuf;
static volatile int16_t RxRdPtr, RxWrtPtr;
static volatile int16_t RxSize;
static volatile int16_t TxRdPtr, TxWrtPtr;
static volatile int16_t TxSize;

/*------------------------------------------------------------------------------
    function prototype
------------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
    UART2_init(void)
------------------------------------------------------------------------------*/

void UART2_init(void)
{
	TxRdPtr = TxWrtPtr = 0;
	TxSize = 0;

	RxRdPtr = RxWrtPtr = 0;
	RxSize = 0;


    /* Enable the UART Data Register not empty Interrupt */
    __HAL_UART_ENABLE_IT(&huart2, UART_IT_RXNE);

    /* Disable the UART Parity Error Interrupt */
    __HAL_UART_DISABLE_IT(&huart2, UART_IT_PE);

    /* Disable the UART Error Interrupt: (Frame error, noise error, overrun error) */
    __HAL_UART_DISABLE_IT(&huart2, UART_IT_ERR);


    /* Disable the UART Transmit Complete Interrupt */
    __HAL_UART_DISABLE_IT(&huart2, UART_IT_TXE);

    /* Disable the UART Transmit Complete Interrupt */
    __HAL_UART_DISABLE_IT(&huart2, UART_IT_TC);

	// UART2 start receive by interrupt
	// HAL_UART_Receive_IT(&huart2, (uint8_t *)&rbuf, 1);
}


/*------------------------------------------------------------------------------
    UART2_interrupt(void)
------------------------------------------------------------------------------*/

void UART2_interrupt(void)
{
  UART_HandleTypeDef *huart;
  char c;
  huart = &huart2;

  uint32_t isrflags   = READ_REG(huart->Instance->SR);
  uint32_t cr1its     = READ_REG(huart->Instance->CR1);
  //uint32_t cr3its     = READ_REG(huart->Instance->CR3);
  uint32_t errorflags = 0x00U;

  /* If error occurs, do nothing */
  errorflags = (isrflags & (uint32_t)(USART_SR_PE | USART_SR_FE | USART_SR_ORE | USART_SR_NE));
  if(errorflags != RESET) return;

  /*--- Receive Interrupt ---*/
  if(((isrflags & USART_SR_RXNE) != RESET) && ((cr1its & USART_CR1_RXNEIE) != RESET)){
    c = (char)(huart->Instance->DR & (uint8_t)0x00FF);
    if(RxSize < RXBUFSIZE){
	  RxBuf[RxWrtPtr]= c;
	  RxWrtPtr++;
	  RxWrtPtr %= RXBUFSIZE;
	  RxSize++;
    }
  }

  /*--- Transmit Interrupt -----*/
  if (((isrflags & USART_SR_TXE) != RESET) && ((cr1its & USART_CR1_TXEIE) != RESET)){
    if(TxSize){
        huart->Instance->DR = TxBuf[TxRdPtr++];
        TxRdPtr %= TXBUFSIZE;
        TxSize--;
    }else{
        __HAL_UART_DISABLE_IT(&huart2, UART_IT_TXE);
    }
  }

#if 0
  /*--- Transmit complete Interrupt ---*/
  if (((isrflags & USART_SR_TC) != RESET) && ((cr1its & USART_CR1_TCIE) != RESET)){
   /* Disable the UART Transmit Complete Interrupt */
    __HAL_UART_DISABLE_IT(&huart2, UART_IT_TC);

    if(TxSize){
    	__HAL_UART_ENABLE_IT(&huart2, UART_IT_TXE);
    }
    //UART_EndTransmit_IT(huart);
  }
#endif

  return;
}


/*------------------------------------------------------------------------------
    UART2_putchar(char c)
------------------------------------------------------------------------------*/

void UART2_putchar(char c)
{
	while(TxSize == TXBUFSIZE);
	TxBuf[TxWrtPtr++]=c;
	TxWrtPtr %= TXBUFSIZE;

	__HAL_UART_DISABLE_IT(&huart2, UART_IT_TXE);
   	TxSize++;
    __HAL_UART_ENABLE_IT(&huart2, UART_IT_TXE);

	//while(huart2.gState != HAL_UART_STATE_READY);
	//TxBuf[0]=c;
    //HAL_UART_Transmit_IT(&huart2, (uint8_t *)&TxBuf, 1);
}


/*------------------------------------------------------------------------------
    UART2_putstring(char *str)
------------------------------------------------------------------------------*/

void UART2_putstring(char *str)
{
	while(*str){
        UART2_putchar(*str);
        str++;
    }

	//int len;

    //len = strlen(str);
    //if(len <TXBUFSIZE){
    //	while(huart2.gState != HAL_UART_STATE_READY);
    //	strcpy(TxBuf, str);
    //	HAL_UART_Transmit_IT(&huart2, (uint8_t *)&TxBuf, len);
    //}
}


/*------------------------------------------------------------------------------
    UART2_putdata(char *data, int16_t len)
------------------------------------------------------------------------------*/

void UART2_putdata(char *data, int16_t len)
{
//    while(len--){
//        UART2_putchar(*data);
//        data++;
//    }
}


/*------------------------------------------------------------------------------
    UART2_putcrlf(void)
------------------------------------------------------------------------------*/

void UART2_putcrlf(void)
{
	UART2_putstring("\r\n");
}


/*------------------------------------------------------------------------------
    UART2_gettxsize(void)
------------------------------------------------------------------------------*/

int16_t UART2_gettxsize(void)
{
 	return TxSize;
}


/*------------------------------------------------------------------------------
    UART2_txbufclr(void)
------------------------------------------------------------------------------*/

void UART2_txbufclr(void)
{
    /* Disable the UART Transmit Complete Interrupt */
     __HAL_UART_DISABLE_IT(&huart2, UART_IT_TXE);
     TxSize = 0;
     TxRdPtr = TxWrtPtr = 0;
     /* Enable the UART Transmit Complete Interrupt */
     __HAL_UART_ENABLE_IT(&huart2, UART_IT_TXE);
     //__HAL_UART_ENABLE_IT(&huart2, UART_IT_TC);
}


/*------------------------------------------------------------------------------
    UART2_TxCpltCallback(void)
------------------------------------------------------------------------------*/

void UART2_TxCpltCallback(void)
{
//	TxSize--;
//  TxRdPtr++;
//  TxRdPtr %= TXBUFSIZE;
//  if(TxSize){ // next byte send
//   	HAL_UART_Transmit_IT(&huart2, (uint8_t *)&(TxBuf[TxRdPtr]), 1);
//  }
}


/*------------------------------------------------------------------------------
    UART2_txbufclr(void)
------------------------------------------------------------------------------*/

void UART2_RxCpltCallback(void)
{
//    if(RxSize < RXBUFSIZE){
//		RxBuf[RxWrtPtr]=rbuf;
//		RxWrtPtr++;
//		RxWrtPtr %= RXBUFSIZE;
//		RxSize++;
//	}
//	HAL_UART_Receive_IT(&huart2, (uint8_t *)&rbuf, 1);
}

char UART2_getchar(void)
{
	char c;

	while(RxSize==0);
	c = RxBuf[RxRdPtr];
	RxRdPtr++;
	RxRdPtr %= RXBUFSIZE;
	__HAL_UART_DISABLE_IT(&huart2, UART_IT_RXNE);
	RxSize--;
	__HAL_UART_ENABLE_IT(&huart2, UART_IT_RXNE);

	return c;
}

int16_t UART2_getrxsize(void)
{
	return RxSize;
}

void UART2_rxbufclr(void)
{
	__HAL_UART_DISABLE_IT(&huart2, UART_IT_RXNE);
    RxSize = 0;
    RxRdPtr = RxWrtPtr = 0;
	__HAL_UART_ENABLE_IT(&huart2, UART_IT_RXNE);
}

void UART2_getstring(char buf[], int buf_len)
{
    int c;
    int len;

    len = 0;
    for(;;){
        c = UART2_getchar();

        if(c == '\r'){
            UART2_putcrlf();
            break;
        }

        if(c == 0x08){
            if(len){
                UART2_putchar(0x08);
                UART2_putchar(' ');
                UART2_putchar(0x08);
                len--;
            }
            continue;
        }

        if(c < ' ' || c > 0x7e){
            continue;
        }

        if(len < buf_len-1){
            UART2_putchar(c);
            buf[len++] = c;
        }
    }
    buf[len] = 0;
}

static char hexstr[] = "0123456789ABCDEF";

void UART2_puthex(uint8_t val)
{
	UART2_putchar(hexstr[(val>>4)&0x0f]);
	UART2_putchar(hexstr[val & 0x0f]);
}

void UART2_putword(uint16_t val)
{
    UART2_puthex((val >> 8)&0xff);
    UART2_puthex(val & 0xff);
}

void UART2_putdword(uint32_t val)
{
    UART2_putword((val >> 16)&0xffff);
    UART2_putword(val & 0xffff);
}

void UART2_putdec(int32_t val)
{
    int divide;
    int digit;
    int zflg;

    if(val == 0x80000000){
        UART2_putstring("-2147483648");
        return;
    }

    if(val < 0){
        UART2_putchar('-');
        val = -val;
    }
    divide = 1000000000;
    zflg=0;

    for(;;){
        if(divide ==1){
        	UART2_putchar(val + '0');
            break;
        }
        digit = val / divide;
        if(digit==0){
            if(zflg){
            	UART2_putchar('0');
            }
        }else{
        	UART2_putchar(digit +'0');
            zflg=1;
        }
        val %= divide;
        divide /= 10;
    }

}


/*** end of file **************************************************************/



